<?php 

$lang['Delete users'] = 'Видалив користувача(ів) "%s"';
$lang['Change pass'] = 'Змінив пароль користувача "%s"';
$lang['Change user data'] = 'Змінив дані користувача "%s"';
$lang['Changed user rules'] = 'Змінив правила доступу для ролі "%s"';
