<?php (defined('BASEPATH')) OR exit('No direct script access allowed');

if (strpos(phpversion(), '7.') !== false) {
    
    require dirname(__FILE__) . '/RT_Router_php_7_2.php';
    
} else {

/* load the MX_Router class */
require realpath('vendor/wiredesignz/hmvc/third_party/MX/Router.php');

class RT_Router extends MX_Router {}

}